/*
Copyright Ideaconsult Ltd. (C) 2005-2007 

Contact: nina@acad.bg

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.
All we ask is that proper credit is given for our work, which includes
- but is not limited to - adding the above copyright notice to the beginning
of your source code files, and to any copyright notice that you may distribute
with programs based on this work.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
package toxTree.tree.rules;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.ArrayList;

import org.openscience.cdk.AtomContainer;
import org.openscience.cdk.CDKConstants;
import org.openscience.cdk.DefaultChemObjectBuilder;
import org.openscience.cdk.Molecule;
import org.openscience.cdk.interfaces.IAtomContainer;
import org.openscience.cdk.io.iterator.IIteratingChemObjectReader;
import org.openscience.cdk.io.iterator.IteratingMDLReader;
import org.openscience.cdk.io.iterator.IteratingSMILESReader;
import org.openscience.cdk.tools.HydrogenAdder;
import org.openscience.cdk.tools.MFAnalyser;

import toxTree.core.IDecisionRuleEditor;
import toxTree.exceptions.DecisionMethodException;
import toxTree.exceptions.MolAnalyseException;
import toxTree.io.IteratingDelimitedFileReader;
import toxTree.query.FunctionalGroups;
import toxTree.query.MolAnalyser;
import toxTree.tree.AbstractRule;
import toxTree.ui.tree.rules.RuleStructuresPanel;

/**
 * A rule, which returns true if the query is isomorphic to one of the structures 
 * read from a preconfigured file of a type SDF, SMI, CSV 
 * @author Nina Jeliazkova
 * <b>Modified</b> 2005-9-6
 */
public class RuleStructuresList extends AbstractRule {
	protected File file;
	protected static transient HydrogenAdder hadder = new HydrogenAdder();
	protected transient ArrayList fileCache = null;
	protected transient FileInputStream fStream = null;
	protected boolean useCache = false; //will be true if fixed
	/**
	 * Comment for <code>serialVersionUID</code>
	 */
	private static final long serialVersionUID = 1445819316236082148L;
	/**
	 * 
	 */
	public RuleStructuresList() {
		this(new File("bodymol.sdf"));
		
	}
	public RuleStructuresList(File file) {
		super();
		logger.debug("Will be using file\t",file.getAbsoluteFile());
		String f = file.getPath().toLowerCase();
		if ((file.exists())  && 
			((f.endsWith(".sdf") || f.endsWith(".smi") || f.endsWith(".csv")))) {
			this.file = file;
		} else {
			logger.debug("File does not exists or unsupported format\t",file.getAbsoluteFile());
			this.file = null;
		}
		setExplanation("Returns true if the query is isomorphic to one of the structures loaded from a preconfigured file of a type SDF, SMI, CSV ");
		setTitle("Exact search");
	}	
	
	/**
	 * {@link toxTree.core.IDecisionRule#verifyRule(IAtomContainer)}
	 */
	public boolean verifyRule(IAtomContainer mol) throws DecisionMethodException {
		
		if (file == null) return false;
		MFAnalyser mf = new MFAnalyser(mol);
		String molFormula = mf.getMolecularFormula();
		
		if (useCache && (fileCache != null) && (fileCache.indexOf(molFormula) == -1)) {
			logger.info("Using cached information of file\t",file.getAbsoluteFile(),"\tCompound\t",molFormula,"\tNOT FOUND");
			return false;  
		}
		
		IIteratingChemObjectReader reader;
		String f = file.getPath().toLowerCase();
		fStream = null;
		boolean FOUND = false;
		try {
			fStream = new FileInputStream(file);
			if (f.endsWith(".sdf")) reader = new IteratingMDLReader(fStream,DefaultChemObjectBuilder.getInstance());
			else if (f.endsWith(".csv")) reader = new IteratingDelimitedFileReader(fStream);
			else if (f.endsWith(".smi")) reader = new IteratingSMILESReader(fStream);
			else {
				logger.error("Unsupported format");
				fStream.close(); file = null; return false; }
			
			int r = 0;
			if (fileCache == null) {
				fileCache = new ArrayList();
			}
			
			while (reader.hasNext()) {
				String formula = "";
				r++;
				Object o = reader.next();
				
				if ((r-1) < fileCache.size()) {
					formula = (String)fileCache.get(r-1);
					/*if cached formula of this record does not match our formula,
					 * skip analysis of this record and try the next
					 */
					if (useCache && !molFormula.equals(formula))  
						continue;
				} else fileCache.add(formula);
				
				if (o instanceof AtomContainer) {

					Molecule m = (Molecule) o;
					try {
						MolAnalyser.analyse(m);
						if (formula.equals("")) 
							fileCache.set(r-1,mf.analyseAtomContainer(m));
						
						/*
						m.setFlag(CDKConstants.ISAROMATIC,HueckelAromaticityDetector.detectAromaticity(m,true));			
						hadder.addExplicitHydrogensToSatisfyValency(m);
						*/
						if (m.getAtomCount() != mol.getAtomCount()) {
							continue;
						}
						if (m.getBondCount() != mol.getBondCount()) {
							continue;
						}
						
						if (m.getFlag(CDKConstants.ISAROMATIC) != mol.getFlag(CDKConstants.ISAROMATIC)) {
							logger.debug("Aromaticity does not match");
							continue;							
						}
					
					} catch (MolAnalyseException x) {
						//hmm
						logger.error(x);
					}					
					if (FunctionalGroups.isSubstance(m,mol)) {
						logger.info("Found");
						
						FOUND = true;
						
						if (useCache) {
							reader.close();
							return true;
						}
						//else read the entire file
					}
				}
			}
			if (!FOUND) logger.info("Not found");
			reader.close();
			useCache = fileCache.size() > 0;
			/*
			for (int i=0;i<fileCache.size();i++) 
				System.out.println(fileCache.get(i));
				*/			
			return FOUND;
		} catch (IOException x) {
			logger.error(x.getMessage());
			throw new DecisionMethodException(x);
		}

		
	}
	/* (non-Javadoc)
	 * @see toxTree.tree.AbstractRule#isImplemented()
	 */
	@Override
	public boolean isImplemented() {
		 
		return (file !=null) && (file.exists());
	}

	/**
	 * @return Returns the file.
	 */
	public synchronized File getFile() {
		return file;
	}
	/**
	 * @param file The file to set.
	 */
	public synchronized void setFile(File file) {
		if (!this.file.equals(file)) {
			if (fileCache != null) fileCache.clear();
			fileCache = null;
		}
		this.file = file;
		
	}
	/**
	 * @return Returns the useCache.
	 */
	public synchronized boolean isUsingCache() {
		return useCache;
	}
	/**
	 * @param useCache The useCache to set.
	 */
	public synchronized void setUsingCache(boolean useCache) {
		this.useCache = useCache;
	}
	@Override
	public IDecisionRuleEditor getEditor() {
		return new RuleStructuresPanel(this);
	}
}
/*
class RecordCache implements Comparable {
	protected long offset= 0;
	protected long recNum = 0;
	protected String formula = "";
	protected boolean aromatic = false;
	
	public RecordCache(long offset, long recNum, String formula) {
		super();
		this.offset = offset;
		this.recNum = recNum;
		this.formula = formula;
	}
	public RecordCache(long offset, long recNum) {
		super();
		this.offset = offset;
		this.recNum = recNum;
	
	}	
	
	public boolean equals(Object obj) {
		RecordCache o = (RecordCache) obj;
		return formula.equals(o.formula); 
			
	}
	
	public int compareTo(Object o) {
		return formula.compareTo(((RecordCache) o).formula);
	}
	
	
	
	public synchronized String getFormula() {
		return formula;
	}

	public synchronized void setFormula(String formula) {
		this.formula = formula;
	}
	
	public synchronized boolean isAromatic() {
		return aromatic;
	}

	public synchronized void setAromatic(boolean aromatic) {
		this.aromatic = aromatic;
	}
}

*/
